#include <u.h>
#include <libc.h>
#include <bio.h>
#include <draw.h>
#include <html.h>
#include <ctype.h>
#include "dat.h"

enum {
	Verbatim = 0,			/* layout preserved, no line wrap, no justification */
};

typedef struct Xlate Xlate;
struct Xlate {
	int width;			/* width of this block (may be a table cell) */
	int just;			/* justification mode l/r/c/0 0==Verbatim */
	int pos;			/* output position (column) */
};

static void walk(Xlate *x, Item *items);

/* write text blocks */
static void
text(Xlate *x, Item *item)
{
	Itext *it;

	if(item->state & IFwrap)
		x->width = -1;

	it = (Itext *)item;

	if(item->state & IFbrksp){
		lnbrk(x);
		chr(x, '\n');
	}
	else
	if(tem->state & (IFbrk|IFbrksp)){
		if(x->pos > x->width)
			x->width = x->pos;
		x->pos = 0;
	}

	if(item->state & IFwrap)
		x->width = -1;
	p = str;
	if((e = strchr(p, '\n')) == nil){
		width = strlen(p);
	}
	else{
		width = 0;
		do{
			*e = 0;
			if(e-p > width)
				width = e-p;
			p = e+1;
		}while((e = strchr(p, '\n')) != nil);
	}
	else
		x->pos += runestrlen(it->s);
}

/* find the width of the text in a cell */
static int
cellwidth(Xlate *x, Item *item)
{
	int width;
	Xlate xlate;
	char *p, *e, *str;

	xlate = *x;
	xlate.f = &xlate.fmt;
	xlate.width = x->width;
	xlate.pos = 0;

	fmtstrinit(xlate.f);
	walk(&xlate, (Itext *)item);
	str = fmtstrflush(xlate.f);

	p = str;
	if((e = strchr(p, '\n')) == nil){
		width = strlen(p);
	}
	else{
		width = 0;
		do{
			*e = 0;
			if(e-p > width)
				width = e-p;
			p = e+1;
		}while((e = strchr(p, '\n')) != nil);
	}
	free(str);
	return width;
}

/* count the width of the text in a cell, zero if none, or -1 if wrapped */
static void
cellwidth(Xlate *x, Item *items)
{
	int i;
	Item *il;
	Ispacer *is;
	char *href;

	for(il = items; il != nil; il = il->next){
		switch(il->tag){
		case Itexttag:
			text(x, il);
			break;
		case Iruletag:
			break;
		case Iformfieldtag:
			break;
		case Itabletag:
			break;
		case Ifloattag:
			cellwidth(x, ((Ifloat*)il)->item);
			break;
		case Iimagetag:
			image(x, il);
			break;
		case Ispacertag:
			is = (Ispacer*)il;
			switch(is->spkind){
			case ISPvline:
				if(x->just == Verbatim)
					chr(x, '\n');
				break;
			case ISPhspace:
				chr(x, ' ');
				break;
			case ISPgeneral:
				chr(x, ' ');
				break;
			case ISPnull:
				chr(x, ' ');
				break;
			}
			break;
		default:
			fprint(2, "%s: libhtml error, unknown item tag %d\n", argv0, il->tag);
		}
	}
}

